/*
  Copyright (c) 2008-2009, Dominik Reichl <dominik.reichl@t-online.de>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "KeyProviderTest.h"
#include <tchar.h>
#include <assert.h>
#include <vector>

static std::vector<KP_KEYPROV_INFO> g_vKeyProv;
static DWORD g_dwIterPos = 0;

static IKpAPI* g_pAPI = NULL;

KP_IMPL_STDREFIMPL(CKeyProviderPluginImpl)

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	UNREFERENCED_PARAMETER(hinstDLL);
	UNREFERENCED_PARAMETER(fdwReason);
	UNREFERENCED_PARAMETER(lpvReserved);

	return TRUE;
}

KP_EXPORT HRESULT KP_API KP_I_INITIALIZELIB_DECL(IKpUnknown* pAPI)
{
	UNREFERENCED_PARAMETER(pAPI);
	return S_OK;
}

KP_EXPORT HRESULT KP_API KP_I_RELEASELIB_DECL(IKpUnknown* pAPI)
{
	UNREFERENCED_PARAMETER(pAPI);
	SAFE_RELEASE(g_pAPI);
	return S_OK;
}

KP_EXPORT HRESULT KP_API KP_I_CREATEINSTANCE_DECL(REFIID riid, void** ppvObject,
	IKpUnknown* pAPI)
{
	*ppvObject = NULL;

	if(pAPI == NULL) return E_UNEXPECTED; // Require API

	if((riid == IID_IKpUnknown) || (riid == IID_IKpPlugin))
	{
		if(pAPI->QueryInterface(IID_IKpAPI, (void**)&g_pAPI) != S_OK)
			return E_FAIL;

		IKpPlugin* pImpl = new CKeyProviderPluginImpl();
		*ppvObject = ((riid == IID_IKpPlugin) ? pImpl : (IKpUnknown*)pImpl);
		return S_OK;
	}

	return E_NOINTERFACE;
}

CKeyProviderPluginImpl::CKeyProviderPluginImpl()
{
	KP_IMPL_CONSTRUCT;

	g_vKeyProv.clear();
	_ProvAdd(_T("Certificate"), 10);
	_ProvAdd(_T("Smart Card"), 20);
	_ProvAdd(_T("User Credentials"), 9);
}

STDMETHODIMP CKeyProviderPluginImpl::QueryInterface(REFIID riid, void** ppvObject)
{
	KP_SUPPORT_INTERFACE(IID_IKpUnknown, IKpUnknown);
	KP_SUPPORT_INTERFACE(IID_IKpPlugin, IKpPlugin);

	*ppvObject = NULL;
	return E_NOINTERFACE;
}

STDMETHODIMP_(BOOL) CKeyProviderPluginImpl::OnMessage(DWORD dwCode, LPARAM lParamW,
	LPARAM lParamL)
{
	UNREFERENCED_PARAMETER(lParamW);
	UNREFERENCED_PARAMETER(lParamL);

	switch(dwCode)
	{
	case KPM_DIRECT_CONFIG:
		MessageBox(NULL, _T("This plugin doesn't have any options."),
			KPT_PLUGIN_NAME, MB_OK | MB_ICONINFORMATION);
		break;

	case KPM_PLUGIN_INFO:
		MessageBox(NULL, _T("Just a sample plugin to demonstrate how a basic key provider plugin looks like."),
			KPT_PLUGIN_NAME, MB_OK | MB_ICONINFORMATION);
		break;

	case KPM_KEYPROV_QUERY_INFO_FIRST:
		g_dwIterPos = 0;
		*((KP_KEYPROV_INFO *)lParamL) = g_vKeyProv[0];
		break;

	case KPM_KEYPROV_QUERY_INFO_NEXT:
		++g_dwIterPos;
		if(g_dwIterPos == 1)
			*((KP_KEYPROV_INFO *)lParamL) = g_vKeyProv[1];
		else if(g_dwIterPos == 2)
			*((KP_KEYPROV_INFO *)lParamL) = g_vKeyProv[2];
		break;

	case KPM_KEYPROV_QUERY_KEY:
		_QueryProvKey((LPCTSTR)lParamW, (KP_KEYPROV_KEY *)lParamL);
		break;

	case KPM_KEYPROV_FINALIZE:
		// Nothing to clean up
		break;

	default:
		break;
	}

	return TRUE;
}

STDMETHODIMP_(LPCTSTR) CKeyProviderPluginImpl::GetProperty(LPCTSTR lpName)
{
	UNREFERENCED_PARAMETER(lpName);
	return NULL;
}

STDMETHODIMP CKeyProviderPluginImpl::SetProperty(LPCTSTR lpName, LPCTSTR lpValue)
{
	UNREFERENCED_PARAMETER(lpName);
	UNREFERENCED_PARAMETER(lpValue);
	return E_FAIL;
}

STDMETHODIMP_(DWORD) CKeyProviderPluginImpl::GetMenuItemCount()
{
	return 0;
}

STDMETHODIMP_(KP_MENU_ITEM*) CKeyProviderPluginImpl::GetMenuItems()
{
	return NULL;
}

void _ProvAdd(LPCTSTR lpProvDisplayName, DWORD dwImageIndex)
{
	KP_KEYPROV_INFO c;
	ZeroMemory(&c, sizeof(KP_KEYPROV_INFO));

	c.lpName = lpProvDisplayName;
	c.dwImageIndex = dwImageIndex;

	g_vKeyProv.push_back(c);
}

void _QueryProvKey(LPCTSTR lpProvDisplayName, KP_KEYPROV_KEY* lpKey)
{
	// Some sample keys. In a real key provider plugin, the key
	// would be retrieved from smart card, USB device, ...
	static BYTE vKey1[4] = { 1, 2, 3, 4 };
	static BYTE vKey2[8] = { 2, 4, 6, 8, 10, 12, 14, 16 };
	static BYTE vKey3[10] = { 2, 3, 5, 7, 11, 13, 17, 19, 23, 29 };

	if(_tcscmp(g_vKeyProv[0].lpName, lpProvDisplayName) == 0)
	{
		lpKey->lpData = &vKey1[0];
		lpKey->dwDataSize = 4;
	}
	else if(_tcscmp(g_vKeyProv[1].lpName, lpProvDisplayName) == 0)
	{
		lpKey->lpData = &vKey2[0];
		lpKey->dwDataSize = 8;
	}
	else if(_tcscmp(g_vKeyProv[2].lpName, lpProvDisplayName) == 0)
	{
		lpKey->lpData = &vKey3[0];
		lpKey->dwDataSize = 10;
	}
}
